import { ChangeDetectorRef, Directive, Input, OnDestroy, OnInit, TemplateRef, ViewContainerRef } from '@angular/core';
import { createRenderAware } from '../core';
import { DEFAULT_STRATEGY_NAME, getStrategies } from '../render-strategies/strategies/strategies-map';
import { TemplateManager } from '../core/utils/template-manager';
/**
 * @Directive LetDirective
 *
 * @description
 *
 * The `*rxLet` directive serves a convenient way of binding observables to a view context (a dom element scope).
 * It also helps with several internal processing under the hood.
 *
 * The current way of binding an observable to the view looks like that:
 * ```html
 * <ng-container *ngIf="observableNumber$ as n">
 * <app-number [number]="n">
 * </app-number>
 * <app-number-special [number]="n">
 * </app-number-special>
 * </ng-container>
 *  ```
 *
 *  The problem is `*ngIf` is also interfering with rendering and in case of a `0` the component would be hidden
 *
 * Included Features:
 * - binding is always present. (`*ngIf="truthy$"`)
 * - it takes away the multiple usages of the `async` or `push` pipe
 * - a unified/structured way of handling null and undefined
 * - triggers change-detection differently if `zone.js` is present or not (`ChangeDetectorRef.detectChanges` or
 *   `ChangeDetectorRef.markForCheck`)
 * - triggers change-detection differently if ViewEngine or Ivy is present (`ChangeDetectorRef.detectChanges` or
 *   `ɵdetectChanges`)
 * - distinct same values in a row (distinctUntilChanged operator),
 *
 * @usageNotes
 *
 * The `*rxLet` directive take over several things and makes it more convenient and save to work with streams in the
 *   template
 * `<ng-container *rxLet="observableNumber$ as c"></ng-container>`
 *
 * ```html
 * <ng-container *rxLet="observableNumber$ as n">
 * <app-number [number]="n">
 * </app-number>
 * </ng-container>
 *
 * <ng-container *rxLet="observableNumber$; let n">
 * <app-number [number]="n">
 * </app-number>
 * </ng-container>
 * ```
 *
 * In addition to that it provides us information from the whole observable context.
 * We can track the observables:
 * - next value
 * - error value
 * - complete base-state
 *
 * ```html
 * <ng-container *rxLet="observableNumber$; let n; let e = $error, let c = $complete">
 * <app-number [number]="n"  *ngIf="!e && !c">
 * </app-number>
 * <ng-container *ngIf="e">
 * There is an error: {{e}}
 * </ng-container>
 * <ng-container *ngIf="c">
 * Observable completed: {{c}}
 * </ng-container>
 * </ng-container>
 * ```
 *
 * @publicApi
 */
export class LetDirective {
    constructor(cdRef, nextTemplateRef, viewContainerRef) {
        this.nextTemplateRef = nextTemplateRef;
        this.viewContainerRef = viewContainerRef;
        this.resetObserver = {
            next: () => {
                this.templateManager.updateViewContext({
                    $implicit: undefined,
                    rxLet: undefined,
                    $error: false,
                    $complete: false
                });
            }
        };
        this.updateObserver = {
            next: (value) => {
                this.templateManager.insertEmbeddedView('rxNext');
                this.templateManager.updateViewContext({
                    $implicit: value,
                    rxLet: value
                });
            },
            error: (error) => {
                // fallback to rxNext when there's no template for rxError
                this.templateManager.insertEmbeddedView('rxNext');
                this.templateManager.insertEmbeddedView('rxError');
                this.templateManager.updateViewContext({
                    $error: true
                });
            },
            complete: () => {
                // fallback to rxNext when there's no template for rxComplete
                this.templateManager.insertEmbeddedView('rxNext');
                this.templateManager.insertEmbeddedView('rxComplete');
                this.templateManager.updateViewContext({
                    $complete: true
                });
            }
        };
        this.strategies = getStrategies({ cdRef });
        this.templateManager = new TemplateManager(this.viewContainerRef, {
            $implicit: undefined,
            rxLet: undefined,
            $error: false,
            $complete: false
        });
        this.renderAware = createRenderAware({
            strategies: this.strategies,
            resetObserver: this.resetObserver,
            updateObserver: this.updateObserver
        });
        this.renderAware.nextStrategy(DEFAULT_STRATEGY_NAME);
    }
    set rxLet(potentialObservable) {
        this.renderAware.nextPotentialObservable(potentialObservable);
    }
    set strategy(strategy) {
        this.renderAware.nextStrategy(strategy || DEFAULT_STRATEGY_NAME);
    }
    set rxLetComplete(templateRef) {
        this.templateManager.addTemplateRef('rxComplete', templateRef);
    }
    set rxLetError(templateRef) {
        this.templateManager.addTemplateRef('rxError', templateRef);
    }
    set rxLetSuspense(templateRef) {
        this.templateManager.addTemplateRef('rxSuspense', templateRef);
    }
    static ngTemplateContextGuard(dir, ctx) {
        return true;
    }
    ngOnInit() {
        this.templateManager.insertEmbeddedView('rxSuspense');
        this.templateManager.addTemplateRef('rxNext', this.nextTemplateRef);
        this.subscription = this.renderAware.subscribe();
    }
    ngOnDestroy() {
        this.subscription.unsubscribe();
        this.templateManager.destroy();
    }
}
LetDirective.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: TemplateRef },
    { type: ViewContainerRef }
];
LetDirective.decorators = [
    { type: Directive, args: [{
                selector: '[rxLet]',
                exportAs: 'renderNotifier'
            },] }
];
LetDirective.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: TemplateRef },
    { type: ViewContainerRef }
];
LetDirective.propDecorators = {
    rxLet: [{ type: Input }],
    strategy: [{ type: Input, args: ['rxLetStrategy',] }],
    rxLetComplete: [{ type: Input }],
    rxLetError: [{ type: Input }],
    rxLetSuspense: [{ type: Input }]
};
//# sourceMappingURL=let.directive.js.map